@@ -1,5 +1,6 @@
 #'@import rhandsontable
 #'@import shiny
+#'@import formods
 #'@importFrom digest digest
 #'@importFrom shinyAce aceEditor updateAceEditor
 
@@ -502,7 +503,7 @@
 #'@examples
 #' # Within shiny both session and input variables will exist,
 #' # this creates examples here for testing purposes:
-#' sess_res = ===ZZ===_test_mksession(session=list())
+#' sess_res = ===ZZ===_test_mksession()
 #' session = sess_res$session
 #' input   = sess_res$input
 #'
@@ -713,7 +714,7 @@
 #'@examples
 #' # Within shiny both session and input variables will exist,
 #' # this creates examples here for testing purposes:
-#' sess_res = ===ZZ===_test_mksession(session=list())
+#' sess_res = ===ZZ===_test_mksession()
 #' session = sess_res$session
 #' input   = sess_res$input
 #'
@@ -818,7 +819,7 @@ code}
 #'  \item{msgs:}      Messages to be passed back to the user.
 #'  \item{rpt:}       Report with any additions passed back to the user.
 #'}
-#'@seealso \code{\link[formods]{FM_generate_report}}
+#'@seealso \code{\link[formods:FM_generate_report]{formods::FM_generate_report()}}
 ===ZZ===_append_report = function(state, rpt, rpttype, gen_code_only=FALSE){
 
   isgood    = TRUE
@@ -858,9 +859,10 @@ res}
 #'  \itemize{
 #'    \item{label: Text label for the dataset}
 #'    \item{MOD_TYPE: Short name for the type of module.}
-#'    \item{id: module ID}
+#'    \item{id: module ID.}
+#'    \item{idx: unique numerical ID to identify this dataset in the module.}
 #'    \item{DS: Dataframe containing the actual dataset.}
-#'    \item{DSMETA: Metadata describing DS}
+#'    \item{DSMETA: Metadata describing DS.}
 #'    \item{code: Complete code to build dataset.}
 #'    \item{checksum: Module checksum.}
 #'    \item{DSchecksum: Dataset checksum.}
@@ -868,7 +870,7 @@ res}
 #'}
 #'@examples
 #' # We need a module state:
-#' sess_res = ===ZZ===_test_mksession(session=list())
+#' sess_res = ===ZZ===_test_mksession()
 #' state = sess_res$state
 #'
 #' ds = ===ZZ===_fetch_ds(state)
@@ -884,6 +886,7 @@ res}
   NEWDS = list(label      = NULL,
                MOD_TYPE   = NULL,
                id         = NULL,
+               idx        = NULL,
                DS         = NULL,
                DSMETA     = NULL,
                code       = NULL,
@@ -922,21 +925,22 @@ res}
 #'    \item{label:}         Text label for the model (e.g. one-compartment model).
 #'    \item{MOD_TYPE:}      Type of module.
 #'    \item{id:}            Module ID.
+#'    \item{idx:}           Numeric ID for the element.
 #'    \item{rx_obj:}        The rxode2 object.
 #'    \item{rx_obj_name:}   The rxode2 object name that holds the model.
 #'    \item{ts_obj:}        List of timescale information for the system and
 #'                          details of other timescales (\code{list(system="weeks", details = list(days=list(verb="days", conv=86400)))})
 #'    \item{ts_obj_name:}   The object name that holds the timescale for this  model.
-#'    \item{fcn_def:}    Text to define the model.
-#'    \item{MDLMETA:}    Notes about the model.
-#'    \item{code:}       Code to generate the model.
-#'    \item{checksum:}   Module checksum.
+#'    \item{fcn_def:}      Text to define the model.
+#'    \item{MDLMETA:}      Notes about the model.
+#'    \item{code:}         Code to generate the model.
+#'    \item{checksum:}     Module checksum.
 #'    \item{MDLchecksum:} Model checksum.
 #'  }
 #'}
 #'@examples
 #' # We need a module state:
-#' sess_res = ===ZZ===_test_mksession(session=list(), full_session=FALSE)
+#' sess_res = ===ZZ===_test_mksession()
 #' state = sess_res$state
 #'
 #' mdls = ===ZZ===_fetch_mdl(state)
@@ -970,6 +974,7 @@ res}
           list(label       = ce[["ui"]][["element_name"]],
                MOD_TYPE    = "===ZZ===",
                id          = state[["id"]],
+               idx         = ce[["idx"]],
                rx_obj      = NULL, #
                rx_obj_name = NULL, #
                ts_obj      = NULL, #
@@ -1035,44 +1040,18 @@ state}
 #'@title Populate Session Data for Module Testing
 #'@description Populates the supplied session variable for testing.
 #'@param session Shiny session variable (in app) or a list (outside of app)
-#'@param id An ID string that corresponds with the ID used to call the modules UI elements
-#'@param full_session  Boolean to indicate if the full test session should be created (default \code{TRUE}).
-#'@return list with the following elements
-#' \itemize{
-#'   \item{isgood:} Boolean indicating the exit status of the function.
-#'   \item{session:} The value Shiny session variable (in app) or a list (outside of app) after initialization.
-#'   \item{input:} The value of the shiny input at the end of the session initialization.
-#'   \item{state:} App state.
-#'   \item{rsc:} The \code{react_state} components.
-#'}
+#'@return The ===ZZ=== portion of the `all_sess_res` returned from \code{\link{ASM_set_app_state}}
 #'@examples
-#' sess_res = ===ZZ===_test_mksession(session=list())
-===ZZ===_test_mksession = function(session, id = "===ZZ===", full_session=TRUE){
+#' session = shiny::MockShinySession$new()
+#' sess_res = ===ZZ===_test_mksession(session=session)
+===ZZ===_test_mksession = function(session = list()){
 
-  isgood = TRUE
-  rsc    = list()
-  input  = list()
+  sources = c(system.file(package="formods", "preload", "ASM_preload.yaml"),
+              system.file(package="formods", "preload", "UD_preload.yaml"))
+  res = ASM_set_app_state(session=list(), sources=sources)
+  res = res[["all_sess_res"]][["===ZZ==="]]
 
-  # Configuration files
-  FM_yaml_file  = system.file(package = "formods", "templates", "formods.yaml")
-  MOD_yaml_file = system.file(package = "===PKG===", "templates", "===ZZ===.yaml")
-
-  # Creating an empty state object
-  state = ===ZZ===_fetch_state(id              = "===ZZ===",
-                         input           = input,
-                         session         = session,
-                         FM_yaml_file    = FM_yaml_file,
-                         MOD_yaml_file   = MOD_yaml_file,
-                         react_state     = NULL)
-
-  res = list(
-    isgood  = isgood,
-    session = session,
-    input   = input,
-    state   = state,
-    rsc     = rsc
-  )
-}
+res}
 
 #'@export
 #'@title New ===Module_Name=== ===ELEMENT===
@@ -1233,9 +1212,9 @@ state}
 #'object to account for differences that may apply between servers.
 #'@param state ===ZZ=== state from \code{===ZZ===_fetch_state()}
 #'@param session Shiny session variable
-#'@return State object with updates made to the state object
+#'@return List contianing the ===ZZ=== state object and shiny session object
 #'@examples
-#' sess_res = ===ZZ===_test_mksession(session=list())
+#' sess_res = ===ZZ===_test_mksession()
 #' session = sess_res$session
 #' state   = sess_res$state
 #' state = ===ZZ===_onload(state, session)
@@ -1245,7 +1224,248 @@ state}
   # have any you can leave this function as a passthrough for the state object
   # or just delete it.
 
-state}
+
+  res =
+  list(state   = state,
+       session = session)
+res}
+
+
+#'@export
+#'@title Preload Data for ===ZZ=== Module
+#'@description Populates the supplied session variable with information from
+#'list of sources.
+#'@param session     Shiny session variable (in app) or a list (outside of app)
+#'@param src_list    List of preload data (all read together with module IDs at the top level)
+#'@param yaml_res    List data from module yaml config
+#'@param mod_ID      Module ID of the module being loaded.
+#'@param react_state Reactive shiny object (in app) or a list (outside of app) used to trigger reactions.
+#'@param quickload   Logical \code{TRUE} to load reduced analysis \code{FALSE} to load the full analysis
+#'@return list with the following elements
+#' \itemize{
+#'   \item{isgood:}      Boolean indicating the exit status of the function.
+#'   \item{msgs:}        Messages to be passed back to the user.
+#'   \item{session:}     Session object
+#'   \item{input:}       The value of the shiny input at the end of the session initialization.
+#'   \item{state:}       App state.
+#'   \item{react_state:} The \code{react_state} components.
+#'}
+===ZZ===_preload  = function(session, src_list, yaml_res, mod_ID=NULL, react_state = list(), quickload=FALSE){
+  isgood  = TRUE
+  input   = list()
+  msgs    = c()
+  res     = c()
+  err_msg = c()
 
 
 
+  FM_yaml_file  = render_str(src_list[[mod_ID]][["fm_yaml"]])
+  MOD_yaml_file = render_str(src_list[[mod_ID]][["mod_yaml"]])
+  id_ASM        = yaml_res[[mod_ID]][["mod_cfg"]][["MC"]][["module"]][["depends"]][["id_ASM"]]
+# id_UD         = yaml_res[[mod_ID]][["mod_cfg"]][["MC"]][["module"]][["depends"]][["id_UD"]]
+# id_DW         = yaml_res[[mod_ID]][["mod_cfg"]][["MC"]][["module"]][["depends"]][["id_DW"]]
+
+  # Creating an empty state object
+  state = ===ZZ===_fetch_state(id              = mod_ID,
+                               id_ASM          = id_ASM,
+                               input           = input,
+                               session         = session,
+                               FM_yaml_file    = FM_yaml_file,
+                               MOD_yaml_file   = MOD_yaml_file,
+                               react_state     = react_state)
+
+  elements = src_list[[mod_ID]][["elements"]]
+
+
+  # Checks to see if we can add elements
+  ADD_ELEMENTS = TRUE
+  if(is.null(elements)){
+    ADD_ELEMENTS = FALSE
+  }
+
+  if(ADD_ELEMENTS){
+    # All of the numeric IDs in the preload
+    enumeric    = c()
+
+    # Map between list index and internal figure ID
+    element_map = list()
+    for(ele_idx in 1:length(elements)){
+      enumeric = c(enumeric, elements[[ele_idx]][["element"]][["idx"]])
+      element_map[[ paste0("element_",elements[[ele_idx]][["element"]][["idx"]] )]] = ele_idx
+    }
+
+    # Creating empty element placeholders
+    while(state[["===ZZ==="]][["element_cntr"]] < max(enumeric)){
+      state = ===ZZ===_new_element(state)
+    }
+
+    # culling any unneeded views
+    for(ele_id  in names(state[["===ZZ==="]][["elements"]])){
+      # This is a view that doesn't exist in elements so
+      # we need to cull it
+      if(!(ele_id  %in% names(element_map))){
+        # Setting the view to be deleted as the current view
+        state[["===ZZ==="]][["elements"]][[ ele_id  ]] = NULL
+      }
+    }
+
+    # TODO: You need to process the elements and components here
+    #browser()
+    # Now we have empty elements defined
+    for(element_id in names(element_map)){
+      # Making the current element id active
+      state[["===ZZ==="]][["current_element"]]  =  element_id
+      ele_err_msg = c()
+
+      # Getting the numeric position in the list corresponding
+      # to the current element id
+      ele_idx = element_map[[element_id]]
+      ele_isgood = TRUE
+
+      #-------------------------------------------------------
+      # Defining general options
+      FM_le(state, paste0("loading element idx: ", ele_idx ))
+
+      # Place checks for required fields here:
+      # req_ele_opts =c("field1", "field2")
+      # if(!all(req_ele_opts    %in% names( elements[[ele_idx]][["element"]]))){
+      #   ele_isgood      = FALSE
+      #   missing_opts    = req_ele_opts[!(req_ele_opts %in% names(elements[[ele_idx]][["element"]]))]
+      #   ele_err_msg = c(ele_err_msg,
+      #     paste0("element idx:  ",ele_idx, " missing option(s):" ),
+      #     paste0("  -> ", paste0(missing_opts, collapse=", "))
+      #     )
+      # }
+
+      # If the module requires components check here:
+      # if(!("components" %in% names(elements[[ele_idx]][["element"]]))){
+      #   ele_isgood = FALSE
+      #   ele_err_msg = c(ele_err_msg,
+      #       paste0("element idx: ",ele_idx, " no components defined"))
+      # }
+
+      # Next we process the components (models)
+      if(ele_isgood){
+
+        # Creating element components
+        # If there are components you can add them here:
+        # for(comp_idx in 1:length(elements[[ele_idx]][["element"]][["components"]])){
+        #   tmp_component = elements[[ele_idx]][["element"]][["components"]][[comp_idx]][["component"]]
+        #   add_component = TRUE
+        #   # If there are required component fields put them here
+        #   # req_comp_opts =c("field1", "field2")
+        #   # if(!all(req_comp_opts    %in% names(tmp_component))){
+        #   #   ele_isgood      = FALSE
+        #   #   add_component   = FALSE
+        #   #   missing_opts    =
+        #   #     req_comp_opts[!(req_comp_opts %in% names(tmp_component))]
+        #   #   ele_err_msg = c(ele_err_msg,
+        #   #     paste0("element idx:  ",ele_idx, ", element idx: ", comp_idx, ", missing option(s):" ),
+        #   #     paste0("  -> ", paste0(missing_opts, collapse=", "))
+        #   #     )
+        #   # }
+        #   if(add_component && ele_isgood){
+        #   }
+        #
+        #
+        # }
+      }
+
+      if(ele_isgood){
+        formods::FM_le(state,paste0("added element idx: ",ele_idx))
+      } else {
+        ele_err_msg = c(
+          paste0("failed to add element idx: ",ele_idx),
+          ele_err_msg)
+        msgs = c(msgs, ele_err_msg)
+        isgood = FALSE
+      }
+    }
+  }
+
+  if(!isgood && !is.null(err_msg)){
+    formods::FM_le(state,err_msg,entry_type="danger")
+    msgs = c(msgs, err_msg)
+  }
+
+
+  formods::FM_le(state,paste0("module isgood: ",isgood))
+
+  if(formods::is_shiny(session)){
+    FM_set_mod_state(session, mod_ID, state)
+  } else {
+    session = FM_set_mod_state(session, mod_ID, state)
+  }
+
+  res = list(isgood      = isgood,
+             msgs        = msgs,
+             session     = session,
+             input       = input,
+             react_state = react_state,
+             state       = state)
+
+res}
+
+#'@export
+#'@title Make List of Current ===ZZ=== State
+#'@description Reads in the app state from yaml files.
+#'@param state ===ZZ=== state object
+#'@return list with the following elements
+#' \itemize{
+#'   \item{isgood:}       Boolean indicating the exit status of the function.
+#'   \item{msgs:}         Messages to be passed back to the user.
+#'   \item{yaml_list:}    Lists with preload components.
+#'}
+#'@examples
+#' sess_res = ===ZZ===_test_mksession()
+#' state = sess_res$state
+#' res = ===ZZ===_mk_preload(state)
+===ZZ===_mk_preload     = function(state){
+  isgood    = TRUE
+  msgs      = c()
+  err_msg   = c()
+  yaml_list = list()
+
+  ylist = list(
+      fm_yaml  = file.path("config", basename(state[["FM_yaml_file"]])),
+      mod_yaml = file.path("config", basename(state[["MOD_yaml_file"]]))
+  )
+
+  # Creating the yaml list with the module ID at the top level
+  yaml_list = list()
+  yaml_list[[ state[["id"]] ]]  = ylist
+
+  ele_idx = 1
+  # Walking through each element:
+  for(element_id in names(state[["===ZZ==="]][["elements"]])){
+    tmp_source_ele = state[["===ZZ==="]][["elements"]][[element_id]]
+
+    FM_le(state, paste0("saving element (", tmp_source_ele[["idx"]], ") ", tmp_source_ele[["ui"]][["element_name"]]))
+
+    tmp_element = list(
+      idx               = tmp_source_ele[["idx"]],
+      name              = tmp_source_ele[["ui"]][["element_name"]],
+      components  = list())
+
+    # Add element and component details here:
+
+    # Appending element
+    ylist[["elements"]][[ele_idx]] = list(element = tmp_element)
+    ele_idx = ele_idx + 1
+  }
+
+  formods::FM_le(state,paste0("mk_preload isgood: ",isgood))
+
+  yaml_list = list()
+  yaml_list[[ state[["id"]] ]]  = ylist
+
+  if(!isgood && !is.null(err_msg)){
+    formods::FM_le(state,err_msg,entry_type="danger")
+    msgs = c(msgs, err_msg)
+  }
+
+  res = list(
+    isgood    = isgood,
+    msgs      = msgs,
+    yaml_list = yaml_list)
+}
