@@ -10,7 +10,9 @@ test_that("FG -- Fetch state", {
 
 test_that("FG -- Forcing builds of figures", {
   for(fid in names(sess_res$state$FG$figs)){
-    expect_equal(class(ggplot2::ggplot_build(sess_res[["state"]][["FG"]][["figs"]][[fid]][["fobj"]])),
-                 "ggplot_built")
+    expect_true(inherits(ggplot2::ggplot_build(sess_res[["state"]][["FG"]][["figs"]][[fid]][["fobj"]]),
+                     c("ggplot2::ggplot_built", "ggplot_built")))
+  # expect_equal(class(ggplot2::ggplot_build(sess_res[["state"]][["FG"]][["figs"]][[fid]][["fobj"]])),
+  #              "ggplot_built")
   }
 })
